#ifndef HEIGHT_WIDTH_H
#define HEIGHT_WIDTH_H

#define BINARY_FILE ".bin"
#define BITMAP_BUTTON_HEIGHT 81
#define BITMAP_BUTTON_WIDTH 72
#define BOX_LIST_SIZE 83
#define CLOSE_BUTTON_HEIGHT 27
#define CLOSE_BUTTON_WIDTH 66
#define FOCUS_HEIGHT 191
#define FOCUS_WIDTH 130
#define FROM_LOG_REPEAT_SETTING 4
#define GROUPING_INDICATOR "-g"
#define HELP_BUTTON_HEIGHT 30
#define MAIN_X 188
#define MAIN_Y 0
#define MANAGE_LIST_BUFFER_SIZE 32
#define MANAGE_LIST_SIZE 8
#define MAPPING_INDICATOR "-m"
#define MARGIN 1
#define MENU_BAR_HEIGHT 19
#define MESSAGE_HEIGHT 259
#define MESSAGE_WIDTH 267
#define NONSAMPLING_PERIOD 1.0
#define NOT_FOUND -1
#define PARAMETER_LIST_SIZE 256
#define PICK_BUTTON_HEIGHT 27
#define PICK_BUTTON_WIDTH 66
#define PRNG_BUTTON_HEIGHT 26
#define PRNG_BUTTON_WIDTH 55
#define SAVE_BUTTON_HEIGHT 27
#define SAVE_BUTTON_WIDTH 66
#define STATUS_BAR_HEIGHT 30
#define STATUS_BAR_LABEL_HEIGHT 7
#define TIME_WRITER_PERIOD 1.0
#define TRANSPARENT_INDICATOR "-w-"
#define UND "und"
#define VERSION_HEIGHT 259
#define VERSION_WIDTH 267
#define VERY_SMALL_BUFFER_SIZE 16
#define VIEW_PDF "kopic.pdf"
#define WEIGHT_HEIGHT 671
#define WEIGHT_WIDTH 281
#define XBM_HEIGHT 40
#define XBM_WIDTH 40
#include <FL/Fl.H>
#include <FL/Fl_Bitmap.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Text_Buffer.H>
#include <FL/Fl_Text_Display.H>
#include <FL/Fl_Timer.H>
#include <FL/Fl_Widget.H>
#include <FL/Fl_Window.H>
#include <FL/x.H>
#include <fstream>
#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <string>
#include "kopicLog.h"
#include "kun.h"
#include "OS-specific.h"

enum AIM {
    NO_AIM,
    CLEAR_BLOCK,
    DRAW,
    END_BLOCK,
    FILL_BLOCK,
    FIRST_UNDO,
    GO,
    LINE,
    START_BLOCK
};

enum REVERSE_COMMAND {
    NO_OPERATION,
    UNDO_CLEAR,
    UNDO_DRAW,
    UNDO_END,
    UNDO_FILL,
    UNDO_GO,
    UNDO_LINE,
    UNDO_START
};

typedef struct UNDO_DATA UNDO_DATA;
struct UNDO_DATA {
    bool previous_item;
    char *unexpanded;
    enum REVERSE_COMMAND reverse_command;
    int cell_x, cell_y, previous_aim, previous_block_x, previous_block_y, previous_x, previous_y;
};

typedef Fl_Button *FL_BUTTON;
typedef Fl_Input *FL_INPUT;
class FocusPane : public Fl_Double_Window {
    const int BUTTON_X, FIELD_HEIGHT, FIELD_WIDTH;
    const std::string COMMA, SEPARATOR;
    FL_BUTTON closeButton, saveButton;
    FL_INPUT input1, input2, input3, input4;
    int data_height, data_width, focus_bottom, focus_left, focus_right, focus_top;

public:
    ~FocusPane(){}
    FocusPane(int W, int H, const char *t) : Fl_Double_Window(W, H, t), BUTTON_X(61), COMMA(","), FIELD_HEIGHT(27), FIELD_WIDTH(56), SEPARATOR(" / ") {
    }

    bool setting(KUN_CODE *p);
    void centering();
    void saveHandler();
};

class MessagePane : public Fl_Double_Window {
    FL_BUTTON bitmapButton, closeButton;

public:
    ~MessagePane(){}
    MessagePane(int W, int H, const char *t) : Fl_Double_Window(W, H, t) {} 

    bool setting(PICK_CODE *p);
    void centering();
    void helpHandler();
};

class VersionPane : public Fl_Double_Window {
    FL_BUTTON bitmapButton, closeButton;

public:
    ~VersionPane(){}
    VersionPane(int W, int H, const char *t) : Fl_Double_Window(W, H, t) {} 

    bool setting();
    void centering();
    void helpHandler();
};

class WeightPane : public Fl_Double_Window {
    bool grouping, mapping, weight;
    const int BUTTON_X, FIELD_HEIGHT, FIELD_WIDTH;
    FL_BUTTON closeButton, MTbutton, pickButton, PRNGbutton, xorshiftButton;
    FL_INPUT indicator, input1, input2, input3, input4, input5, input6, input7, input8, input9, input10, input11, input12, input13, input14, input15, input16, input17, input18, input19, input20, input21, input22, input23, input24, input25, input26, input27, input28, input29, input30, input31, input32, input33, input34, input35, input36, input37, input38, input39, input40, input41;
    int decrement_by_n, session;
    int field[ ALL + FROM_LOG_REPEAT_SETTING + 1 ];

public:
    ~WeightPane(){}
    WeightPane(int W, int H, const char *t);
    PICK_CODE setting(const char *f, KUN_CODE *p);
    PICK_CODE textFields();
    int centering();
    void logHandler();
    void PRNGhandler();
};

class Kopic : public Fl_Window {
    bool block, first_undo, persistent;
    bool doubleRecordUndoClosed[2];
    char full_record[FULL_RECORD_SIZE];
    char record[RECORD_SIZE];
    char unexpanded[KOPIC_KUN_SIZE];
    const int SINGLE_RECORD_UNDO, STATUS_BAR_TEXT_LIMIT, TEXT_LIMIT;
    const std::string COMMA, EMPTY, NEW_LINE, SPACE, ZERO_X;
    Fl_Boxtype diamond_square;
    FL_BUTTON square[XBM_HEIGHT][XBM_WIDTH];
    Fl_Menu_Bar *menu;
    Fl_Text_Buffer *textBuffer;
    Fl_Text_Display *statusBar;
    std::ifstream getDoubleRecordUndo[2];
    int aim, block_x, block_y, cell_height, cell_width, cell_x, cell_y, data_height, data_width, decrement_by_n, diamond, focus_bottom, focus_left, focus_right, focus_top;
    int main_height, main_width, main_x, main_y;
    int recordNumber[2];
    Kun *kun;
    std::ofstream putDoubleRecordUndo[2];
    std::string timeNameText[2];
    UNDO_DATA ud;
    union KUN_SIZE {
        char size_bytes[4];
        int kun_size;
    } ks, no_block;
    union RECORD_NUMBER {
        char number_bytes[4];
        int record_number;
    } r;
    VersionPane *versionPane;

    bool kunExpand();
    bool openDoubleRecordUndo();
    bool readUndoBin();
    void addUndoData(); // UNDO_DATA ud, union KUN_SIZE ks
    void gentleSlope(double slope, double intercept, int from_x, int to_x, int from_y, int to_y);
    void moveDiamond(int new_x, int new_y);
    void placeCursor();
    void placeSuaresAccordingToArray();
    void set_EX_OR_array();
    void startBlock(enum AIM a);
    void steepSlope(double slope, double intercept, int from_x, int to_x, int from_y, int to_y);
    void undoBlock();
    void undoClear();
    void undoDraw();
    void undoEnd();
    void undoFill();
    void undoGo();
    void undoLine();
    void undoStart();

public:
    ~Kopic(){}
    Kopic(int W, int H, const char *t) : Fl_Window(W, H, t) , COMMA(","), EMPTY(""), NEW_LINE("\n"), SINGLE_RECORD_UNDO(0), SPACE(" "), STATUS_BAR_TEXT_LIMIT(80), TEXT_LIMIT(3), ZERO_X("0x") {
        main_height = H;
        main_width = W;
    } 

    bool setting(KUN_CODE *p, VersionPane *v);
    bool undoThat();
    void centering();
    void clearCursor();
    void CmClearBottom();
    void CmClearDown();
    void CmClearEnd();
    void CmClearRight();
    void CmClearSide();
    void CmClearStart();
    void CmCsv();
    void CmDecrement();
    void CmDrawDown();
    void CmDrawEnd();
    void CmDrawInvert();
    void CmDrawLeft();
    void CmDrawRight();
    void CmDrawStart();
    void CmDrawUp();
    void CmFillBottom();
    void CmFillDown();
    void CmFillEnd();
    void CmFillRight();
    void CmFillSide();
    void CmFillStart();
    void CmFocus();
    void CmGoCorner();
    void CmGoDiagonal();
    void CmGoDown();
    void CmGoEnd();
    void CmGoJump();
    void CmGoLeft();
    void CmGoRight();
    void CmGoUp();
    void CmKun();
    void CmLineEnd();
    void CmLineRight();
    void CmLineStart();
    void CmLineTo();
    void CmLineUnder();
    void CmText();
    void CmWeight();
    void CmXbm();
    void firstUndo();
    void focussing(std::string & s, int l, int t, int r, int b);
    void goStart();
    void helpHandler();
    void layout();
    void redraw();
    void setStatusBar();
    void standardHandler();
    void temporaryFileName(char *name);
    void versionHandler();
};

bool kopicCsvTouched();
std::string inheritTerms();
std::string standardTerms();
void button_cb(Fl_Widget *, void *);
void clear_bottom_cb(Fl_Widget *, void *);
void clear_down_cb(Fl_Widget *, void *);
void clear_end_cb(Fl_Widget *, void *);
void clear_right_cb(Fl_Widget *, void *);
void clear_side_cb(Fl_Widget *, void *);
void clear_start_cb(Fl_Widget *, void *);
void close_focus_cb(Fl_Widget *, void *);
void close_message_cb(Fl_Widget *, void *);
void close_weight_cb(Fl_Widget *, void *);
void close_version_cb(Fl_Widget *, void *);
void create_kopic_icon(Fl_Window *window); // OS-specific.cpp
void csv_cb(Fl_Widget *, void *);
void decrement_cb(Fl_Widget *, void *);
void draw_down_cb(Fl_Widget *, void *);
void draw_end_cb(Fl_Widget *, void *);
void draw_invert_cb(Fl_Widget *, void *);
void draw_left_cb(Fl_Widget *, void *);
void draw_right_cb(Fl_Widget *, void *);
void draw_start_cb(Fl_Widget *, void *);
void draw_up_cb(Fl_Widget *, void *);
void expand_cb(Fl_Widget *, void *);
void figures(char *buffer, int value);
void fill_bottom_cb(Fl_Widget *, void *);
void fill_down_cb(Fl_Widget *, void *);
void fill_end_cb(Fl_Widget *, void *);
void fill_right_cb(Fl_Widget *, void *);
void fill_side_cb(Fl_Widget *, void *);
void fill_start_cb(Fl_Widget *, void *);
void focus_cb(Fl_Widget *, void *);
void full_swing_cb(Fl_Widget *, void *);
void go_corner_cb(Fl_Widget *, void *);
void go_diagonal_cb(Fl_Widget *, void *);
void go_down_cb(Fl_Widget *, void *);
void go_end_cb(Fl_Widget *, void *);
void go_jump_cb(Fl_Widget *, void *);
void go_left_cb(Fl_Widget *, void *);
void go_right_cb(Fl_Widget *, void *);
void go_up_cb(Fl_Widget *, void *);
void help_cb(Fl_Widget *, void *);
void kun_cb(Fl_Widget *, void *);
void line_end_cb(Fl_Widget *, void *);
void line_right_cb(Fl_Widget *, void *);
void line_start_cb(Fl_Widget *, void *);
void line_to_cb(Fl_Widget *, void *);
void line_under_cb(Fl_Widget *, void *);
void log_cb(Fl_Widget *, void *);
void pdf_cb(Fl_Widget *, void *);
void PRNG_cb(Fl_Widget *, void *);
void quit_cb(Fl_Widget *, void *);
void redraw_cb(Fl_Widget *, void *);
void standard_cb(Fl_Widget *, void *);
void text_cb(Fl_Widget *, void *);
void timer_cb(void *);
void undo_cb(Fl_Widget *, void *);
void version_cb(Fl_Widget *, void *);
void weight_cb(Fl_Widget *, void *);
void xbm_cb(Fl_Widget *, void *);

#endif // HEIGHT_WIDTH_H
